﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Objects;

namespace PV178.Lecture8.Demo1
{
    class EntityStateDemo
    {
        public static void Run()
        {
            using (var db = new UniversalExportsContext(Program.GetConnectionString()))
            {
                Guid cheeseId = Guid.NewGuid();

                Product cheese = Create(db, cheeseId);

                Modify(db, cheese);

                Delete(db, cheese);

                // ! smazana entita je detached => zakomentovat predchazejici radek
                //Detach(db, cheese);
            }
        }

        private static void Detach(UniversalExportsContext db, Product cheese)
        {
            db.Detach(cheese);

            cheese.Name = "Veverka";

            ListProducts(db);

            db.SaveChanges();

            ListProducts(db);
        }

        private static void ListProducts(UniversalExportsContext db)
        {
            Console.WriteLine("Products table content -----------------------");
            foreach (var p in db.Products)
            {
                Console.WriteLine(p.Name);
            }
            Console.WriteLine(new string('-', 20));
        }

        private static Product Create(UniversalExportsContext db, Guid cheeseId)
        {
            Console.WriteLine("Creating cheese to collection:");
            Product cheese = Product.CreateProduct(cheeseId, "Cheese", 10, 20);

            PrintState("after creation", cheese);

            Console.WriteLine();
            Console.WriteLine("Adding cheese to collection:");

            db.Products.AddObject(cheese);

            PrintState("after add, before save", cheese);

            db.SaveChanges();

            PrintState("after save", cheese);
            return cheese;
        }

        private static void Modify(UniversalExportsContext db, Product cheese)
        {
            Console.WriteLine();
            Console.WriteLine("Modifying cheese:");

            cheese.Name = "Old cheese";

            PrintState("after change, before save", cheese);

            db.SaveChanges();

            PrintState("after save", cheese);
        }

        private static void Delete(UniversalExportsContext db, Product cheese)
        {
            Console.WriteLine();
            Console.WriteLine("Deleting cheese:");

            db.Products.DeleteObject(cheese);

            PrintState("after delete, before save", cheese);

            db.SaveChanges();

            PrintState("after save", cheese);
        }

        private static void PrintState(Product cheese)
        {
            PrintState("", cheese);
        }

        private static void PrintState(string text, Product cheese)
        {
            Console.Write("> Cheese state {1}: ", cheese.EntityState, text);
            Console.ForegroundColor = ConsoleColor.Yellow;
            Console.WriteLine(cheese.EntityState);
            Console.ResetColor();
        }
    }
}
