﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Objects;

namespace PV178.Lecture8.Demo1
{
    class QueryDemo
    {
        public static void Run()
        {
            Prepare();

            LinqToEntitiesQuery();

            // ObjectQuery();

            // EntitySql();
        }

        private static void EntitySql()
        {
            using (var db = new UniversalExportsContext(Program.GetConnectionString()))
            {
                ObjectQuery<Product> qry = db.CreateQuery<Product>(
                    @"SELECT VALUE p FROM Products AS p  WHERE p.UnitsInStock > 10 AND p.Price < @price ORDER BY p.Name");
                qry.Parameters.Add(new ObjectParameter("price", 4500));

                Console.WriteLine("SQL command:");
                Console.WriteLine(qry.ToTraceString());
                Console.WriteLine();

                foreach (var product in qry)
                {
                    Console.WriteLine(product.Name);
                }
            }
        }

        private static void ObjectQuery()
        {
            using (var db = new UniversalExportsContext(Program.GetConnectionString()))
            {
                ObjectQuery<Product> qry = db
                    .Products
                    .Where("it.UnitsInStock > 10 AND it.Price < @price", new ObjectParameter("price", 4500))
                    .OrderBy("it.Name");

                Console.WriteLine("SQL command:");
                Console.WriteLine(qry.ToTraceString());
                Console.WriteLine();

                foreach (var product in qry)
                {
                    Console.WriteLine(product.Name);
                }
            }
        }

        private static void LinqToEntitiesQuery()
        {
            using (var db = new UniversalExportsContext(Program.GetConnectionString()))
            {
                var products = from p in db.Products
                               where p.UnitsInStock > 10 && p.Price < 4500
                               orderby p.Name
                               select new { p.Name, p.Price };

                ObjectQuery q = (ObjectQuery)products;
                Console.WriteLine("SQL command:");
                Console.WriteLine(q.ToTraceString());
                Console.WriteLine();

                foreach (var product in products)
                {
                    Console.WriteLine(product);
                }
            }
        }

        private static void Prepare()
        {
            using (var db = new UniversalExportsContext(Program.GetConnectionString()))
            {
                db.Products.AddObject(Product.CreateProduct(Guid.NewGuid(), "Omega Seamaster Wristwatch ", 4020, 40));
                db.Products.AddObject(Product.CreateProduct(Guid.NewGuid(), "Rolex Wristwatch", 4300, 40));
                db.Products.AddObject(Product.CreateProduct(Guid.NewGuid(), "Night Vision", 5000, 30));
                db.Products.AddObject(Product.CreateProduct(Guid.NewGuid(), "Walther PPK", 14000, 120));
                db.Products.AddObject(Product.CreateProduct(Guid.NewGuid(), "Mini Camera", 1300, 40));
                db.Products.AddObject(Product.CreateProduct(Guid.NewGuid(), "Homing Device ", 200, 10));

                db.SaveChanges();
            }
        }
    }
}
